@@ -135,25 +135,16 @@ void AudioControlPage::DestroyAudioMixer()
 	if (auto* temp = FindWindowByLabel(wxT("Mixer"))) {
 		temp->Enable(false);
 	}
-	if (!m_audioChannels.empty()) {
-		auto* audioSizer = m_audioPanel->GetSizer();
-		for (size_t i = m_audioChannels.size(); i > 0; --i) {
-			audioSizer->Remove(i - 1);
-			auto number = wxString::Format(wxT("%u"), i - 1);
-			delete FindWindowByName(wxT("AudioLabel_")  + number);
-			delete FindWindowByName(wxT("AudioSlider_") + number);
-			delete FindWindowByName(wxT("AudioMode_")   + number);
-		}
-		delete FindWindowByName(wxT("MuteButton"));
+	auto* audioSizer = m_audioPanel->GetSizer();
+	audioSizer->Clear(true);
 
-		auto* noAudio = new wxStaticText(
-			m_audioPanel, -1, wxT("No audio channel data available"),
-			wxDefaultPosition, wxDefaultSize,wxALIGN_CENTRE,
-			wxT("NoAudioText"));
-		audioSizer->Add(noAudio, 1, wxALIGN_CENTER_HORIZONTAL | wxALIGN_CENTER_VERTICAL, 0);
-		audioSizer->Layout();
-		m_audioChannels.clear();
-	}
+	auto* noAudio = new wxStaticText(
+		m_audioPanel, -1, wxT("No audio channel data available"),
+		wxDefaultPosition, wxDefaultSize,wxALIGN_CENTRE,
+		wxT("NoAudioText"));
+	audioSizer->Add(noAudio, 1, wxALIGN_CENTER_HORIZONTAL | wxALIGN_CENTER_VERTICAL, 0);
+	audioSizer->Layout();
+	m_audioChannels.clear();
 }
 
 void AudioControlPage::DisableAudioPanel()
@@ -325,6 +316,7 @@ void AudioControlPage::SetChannelVolume(const wxString& name, const wxString& va
 
 	long intvalue;
 	auto* slider = (wxSlider*)FindWindowById(channel + FIRSTAUDIOSLIDER, this);
+	assert(slider != nullptr);
 	value.ToLong(&intvalue);
 	slider->SetValue(100 - intvalue);
 }
@@ -350,17 +342,21 @@ void AudioControlPage::SetChannelMode(const wxString& name, const wxString& valu
 	//if (value == wxT("off"))    val = wxT("O"); // mute
 	//if (value == wxT("stereo")) val = wxT("S");
 	auto* combo = (wxComboBox*)FindWindowById(channel + FIRSTAUDIOCOMBO, this);
+	assert(combo != nullptr);
 	combo->SetSelection(combo->FindString(val));
 }
 
 void AudioControlPage::InitAudioIO()
 {
+	// TODO: handle multiple connectors of the same class ("midi in"/"midi out")
 	bool available = false;
 	for (auto& connector : m_controller.GetConnectors()) {
 		const wxArrayString& pluggables   = m_controller.GetPluggables();
 		const wxArrayString& descriptions = m_controller.GetPluggableDescriptions();
-		if (connector == wxT("msx-midi-in")) {
+		const wxArrayString& pluggableClasses = m_controller.GetPluggableClasses();
+		if (m_controller.GetConnectorClass(connector) == wxT("midi in")) {
 			available = true;
+			m_midiInConnector = connector;
 			if (auto* child = (wxComboBox*)FindWindowByName(wxT("MidiInSelector"))) {
 				child->Enable(true);
 				child->Clear();
@@ -372,20 +368,22 @@ void AudioControlPage::InitAudioIO()
 				midibutton->Enable(true);
 				child->Append(wxT("--empty--"));
 				for (unsigned j = 0; j < pluggables.GetCount(); ++j) {
-					if (pluggables[j].Lower().StartsWith(wxT("midi-in"))) {
-						if (pluggables[j] == wxT("midi-in-reader")) {
-							child->Append(pluggables[j]);
-						} else {
+					if (pluggableClasses[j] == wxT("midi in")) {
+						// use descriptions instead of names if names are meaningless (Windows?)
+						if (pluggables[j].Lower().StartsWith(wxT("midi-in")) &&
+								pluggables[j] != wxT("midi-in-reader")) {
 							child->Append(descriptions[j]);
+						} else {
+							child->Append(pluggables[j]);
 						}
 					}
 				}
-				child->SetValue(wxT("--empty--"));
-
+				child->SetValue(m_controller.GetConnectorPlugged(connector));
 			}
 		}
-		if (connector == wxT("msx-midi-out")) {
+		if (m_controller.GetConnectorClass(connector) == wxT("midi out")) {
 			available = true;
+			m_midiOutConnector = connector;
 			if (auto* child = (wxComboBox *)FindWindowByName (wxT("MidiOutSelector"))) {
 				child->Enable(true);
 				child->Clear();
@@ -397,15 +395,17 @@ void AudioControlPage::InitAudioIO()
 				midibutton->Enable(true);
 				child->Append(wxT("--empty--"));
 				for (unsigned j = 0; j < pluggables.GetCount(); ++j) {
-					if (pluggables[j].Lower().StartsWith(wxT("midi-out"))) {
-						if (pluggables[j] == wxT("midi-out-logger")) {
-							child->Append(pluggables[j]);
-						} else {
+					if (pluggableClasses[j] == wxT("midi out")) {
+						// use descriptions instead of names if names are meaningless (Windows Soft Synths)
+						if (pluggables[j].Lower().StartsWith(wxT("midi-out")) &&
+								pluggables[j] != wxT("midi-out-logger")) {
 							child->Append(descriptions[j]);
+						} else {
+							child->Append(pluggables[j]);
 						}
 					}
 				}
-				child->SetValue(wxT("--empty--"));
+				child->SetValue(m_controller.GetConnectorPlugged(connector));
 			}
 		}
 		if (connector == wxT("pcminput")) {
@@ -425,11 +425,11 @@ void AudioControlPage::InitAudioIO()
 						child->Append(pluggables[j]);
 					}
 				}
-				child->SetValue(wxT("--empty--"));
+				child->SetValue(m_controller.GetConnectorPlugged(connector));
 			}
 		}
 	}
-	if (available){
+	if (available) {
 		if (auto* temp = FindWindowByLabel(wxT("Sound I/O"))) {
 			temp->Enable(true);
 		}
@@ -440,21 +440,24 @@ void AudioControlPage::OnChangeMidiInPlug(wxCommandEvent& event)
 {
 	auto* box = (wxComboBox*)event.GetEventObject();
 	wxString value = box->GetValue();
+	const wxString connectorName = utils::tclEscapeWord(m_midiInConnector);
 	if (value == wxT("--empty--")) {
-		m_controller.WriteCommand(wxT("unplug msx-midi-in"));
+		m_controller.WriteCommand(wxT("unplug ") + connectorName);
 		return;
 	}
 	const wxArrayString& pluggables   = m_controller.GetPluggables();
 	const wxArrayString& descriptions = m_controller.GetPluggableDescriptions();
+	const wxArrayString& pluggableClasses = m_controller.GetPluggableClasses();
 	for (unsigned i = 0; i < pluggables.GetCount(); ++i) {
-		if ((value == descriptions[i]) && pluggables[i].Contains(wxT("midi-in"))) {
+		if ((value == descriptions[i]) && (pluggableClasses[i] == wxT("midi in"))) {
 			value = pluggables[i];
 		}
 	}
-	m_controller.WriteCommand(wxT("plug msx-midi-in ") + value,
+	const wxString pluggableName = utils::tclEscapeWord(value);
+	m_controller.WriteCommand(wxT("plug ") + connectorName + wxT(" ") + pluggableName,
 		nullptr,
 		[&](const wxString& c, const wxString&) {
-			if (c == wxT("plug msx-midi-in midi-in-reader")) {
+			if (c == (wxT("plug ") + utils::tclEscapeWord(m_midiInConnector) + wxT(" midi-in-reader"))) {
 				InvalidMidiInReader();
 			}
 		});
@@ -466,28 +469,32 @@ void AudioControlPage::InvalidMidiInReader()
 	             wxT("Error"));
 	auto* box = (wxComboBox*)FindWindowByName(wxT("MidiInSelector"));
 	box->SetValue(wxT("--empty--"));
-	m_controller.WriteCommand(wxT("unplug msx-midi-in"));
+	const wxString connectorName = utils::tclEscapeWord(m_midiInConnector);
+	m_controller.WriteCommand(wxT("unplug ") + connectorName);
 }
 
 void AudioControlPage::OnChangeMidiOutPlug(wxCommandEvent& event)
 {
 	auto* box = (wxComboBox*)event.GetEventObject();
 	wxString value = box->GetValue();
+	const wxString connectorName = utils::tclEscapeWord(m_midiOutConnector);
 	if (value == wxT("--empty--")) {
-		m_controller.WriteCommand(wxT("unplug msx-midi-out"));
+		m_controller.WriteCommand(wxT("unplug ") + connectorName);
 		return;
 	}
 	const wxArrayString& pluggables   = m_controller.GetPluggables();
 	const wxArrayString& descriptions = m_controller.GetPluggableDescriptions();
+	const wxArrayString& pluggableClasses = m_controller.GetPluggableClasses();
 	for (unsigned i = 0; i < pluggables.GetCount(); ++i) {
-		if ((value == descriptions[i]) && pluggables[i].Contains(wxT("midi-out"))) {
+		if ((value == descriptions[i]) && (pluggableClasses[i] == wxT("midi out"))) {
 			value = pluggables[i];
 		}
 	}
-	m_controller.WriteCommand(wxT("plug msx-midi-out ") + value,
+	const wxString pluggableName = utils::tclEscapeWord(value);
+	m_controller.WriteCommand(wxT("plug ") + connectorName + wxT(" ") + pluggableName,
 		nullptr,
-		[&](const wxString& c, const wxString&) {
-			if (c == wxT("plug msx-midi-out midi-out-logger")) {
+		[&](const wxString& c, const wxString& err) {
+			if (c == (wxT("plug ") + utils::tclEscapeWord(m_midiOutConnector) + wxT(" midi-out-logger"))) {
 				InvalidMidiOutLogger();
 			}
 		});
@@ -499,7 +506,8 @@ void AudioControlPage::InvalidMidiOutLogger()
 	             wxT("Error"));
 	auto* box = (wxComboBox*)FindWindowByName(wxT("MidiOutSelector"));
 	box->SetValue(wxT("--empty--"));
-	m_controller.WriteCommand(wxT("unplug msx-midi-out"));
+	const wxString connectorName = utils::tclEscapeWord(m_midiOutConnector);
+	m_controller.WriteCommand(wxT("unplug ") + connectorName);
 }
 
 void AudioControlPage::OnChangeSampleInPlug(wxCommandEvent& event)
@@ -536,7 +544,7 @@ void AudioControlPage::OnBrowseMidiInFile(wxCommandEvent& event)
 	if (filedlg.ShowModal() == wxID_OK) {
 		miditext->SetValue (filedlg.GetPath());
 		if (!miditext->GetValue().IsEmpty()) {
-			m_controller.WriteCommand(wxT("set midi-in-readfilename ") + utils::ConvertPath(miditext->GetValue()));
+			m_controller.WriteCommand(wxT("set midi-in-readfilename ") + utils::ConvertPathForCommand(miditext->GetValue()));
 		}
 	}
 }
@@ -549,7 +557,7 @@ void AudioControlPage::OnBrowseMidiOutFile(wxCommandEvent& event)
 	if (filedlg.ShowModal() == wxID_OK) {
 		miditext->SetValue(filedlg.GetPath());
 		if (!miditext->GetValue().IsEmpty()) {
-			m_controller.WriteCommand(wxT("set midi-out-logfilename ") + utils::ConvertPath(miditext->GetValue()));
+			m_controller.WriteCommand(wxT("set midi-out-logfilename ") + utils::ConvertPathForCommand(miditext->GetValue()));
 		}
 	}
 }
@@ -562,7 +570,7 @@ void AudioControlPage::OnBrowseSampleInFile(wxCommandEvent& event)
 	if (filedlg.ShowModal() == wxID_OK) {
 		sampletext->SetValue(filedlg.GetPath());
 		if (!sampletext->GetValue().IsEmpty()) {
-			m_controller.WriteCommand(wxT("set audio-inputfilename ") + utils::ConvertPath(sampletext->GetValue()));
+			m_controller.WriteCommand(wxT("set audio-inputfilename ") + utils::ConvertPathForCommand(sampletext->GetValue()));
 		}
 	}
 }
