@@ -23,14 +23,14 @@ BEGIN_EVENT_TABLE(MiscControlPage, wxPanel)
 	EVT_TOGGLEBUTTON  (XRCID("FirmwareButton"),           MiscControlPage::OnFirmware)
 	EVT_COMMAND_SCROLL(XRCID("SpeedSlider"),              MiscControlPage::OnSpeedChange)
 	EVT_BUTTON        (XRCID("NormalSpeedButton"),        MiscControlPage::OnSetNormalSpeed)
-	EVT_TOGGLEBUTTON  (XRCID("MaxSpeedButton"),           MiscControlPage::OnSetMaxSpeed)
-	EVT_COMMAND_SCROLL(XRCID("MinFrameSkipSlider"),       MiscControlPage::OnMinFrameSkipChange)
+	EVT_TOGGLEBUTTON  (XRCID("FastForwardButton"),        MiscControlPage::OnSetFastForward)
+	EVT_COMMAND_SCROLL(XRCID("FastForwardSpeedSlider"),   MiscControlPage::OnFastForwardSpeedChange)
+	EVT_TOGGLEBUTTON  (XRCID("FastLoadingButton"),        MiscControlPage::OnSetFastLoading)
 	EVT_COMMAND_SCROLL(XRCID("MaxFrameSkipSlider"),       MiscControlPage::OnMaxFrameSkipChange)
-	EVT_BUTTON        (XRCID("DefaultMinFrameSkipButton"),MiscControlPage::OnSetDefaultMinFrameSkip)
+	EVT_BUTTON        (XRCID("DefaultFastForwardSpeedButton"),MiscControlPage::OnSetDefaultFastForwardSpeed)
 	EVT_BUTTON        (XRCID("DefaultMaxFrameSkipButton"),MiscControlPage::OnSetDefaultMaxFrameSkip)
-	EVT_TOGGLEBUTTON  (XRCID("ThrottleButton"),           MiscControlPage::OnSetThrottle)
 	EVT_TOGGLEBUTTON  (XRCID("CmdTimingButton"),          MiscControlPage::OnSetCmdTiming)
-	EVT_TEXT          (XRCID("MinFrameSkipIndicator"),    MiscControlPage::OnInputMinFrameskip)
+	EVT_TEXT          (XRCID("FastForwardSpeedIndicator"), MiscControlPage::OnInputFastForwardSpeed)
 	EVT_TEXT          (XRCID("MaxFrameSkipIndicator"),    MiscControlPage::OnInputMaxFrameskip)
 	EVT_TEXT          (XRCID("SpeedIndicator"),           MiscControlPage::OnInputSpeed)
 	EVT_TEXT          (XRCID("Joyport1Selector"),         MiscControlPage::OnChangeJoystick)
@@ -58,30 +58,30 @@ MiscControlPage::MiscControlPage(wxWindow* parent, openMSXController& controller
 	m_speedIndicator = (wxTextCtrl*)FindWindowByName(wxT("SpeedIndicator"));
 	m_speedSlider = (wxSlider*)FindWindowByName(wxT("SpeedSlider"));
 	m_speedNormalButton = (wxButton*)FindWindowByName (wxT("NormalSpeedButton"));
-	m_speedMaxButton = (wxToggleButton*)FindWindowByName (wxT("MaxSpeedButton"));
-	m_minFrameSkipIndicator = (wxTextCtrl*)FindWindowByName(wxT("MinFrameSkipIndicator"));
+	m_fastForwardButton = (wxToggleButton*)FindWindowByName (wxT("FastForwardButton"));
+	m_fastForwardSpeedIndicator = (wxTextCtrl*)FindWindowByName(wxT("FastForwardSpeedIndicator"));
+	m_fastLoadingButton = (wxToggleButton*)FindWindowByName (wxT("FastLoadingButton"));
 	m_maxFrameSkipIndicator = (wxTextCtrl*)FindWindowByName(wxT("MaxFrameSkipIndicator"));
 	m_maxFrameSkipSlider = (wxSlider*)FindWindowByName(wxT("MaxFrameSkipSlider"));
-	m_minFrameSkipSlider = (wxSlider*)FindWindowByName(wxT("MinFrameSkipSlider"));
+	m_fastForwardSpeedSlider = (wxSlider*)FindWindowByName(wxT("FastForwardSpeedSlider"));
 	m_renshaTurboSlider  = (wxSlider*)FindWindowByName(wxT("RenshaTurboSlider"));
 	m_browsePrinterLog = (wxBitmapButton*)FindWindowByName(wxT("BrowsePrinterLog"));
 	m_printerLogFile = (wxTextCtrl*)FindWindowByName(wxT("PrinterLogFile"));
 	m_printerportSelector = (wxComboBox*)FindWindowByName(wxT("PrinterportSelector"));
 
 	m_defaultMaxFrameSkipButton = (wxButton*)FindWindowByName(wxT("DefaultMaxFrameSkipButton"));
-	m_defaultMinFrameSkipButton = (wxButton*)FindWindowByName(wxT("DefaultMinFrameSkipButton"));
+	m_defaultFastForwardSpeedButton = (wxButton*)FindWindowByName(wxT("DefaultFastForwardSpeedButton"));
 
-	m_speedSlider->SetTickFreq(25, 1);
-	m_maxFrameSkipSlider->SetTickFreq(5, 1);
-	m_minFrameSkipSlider->SetTickFreq(5, 1);
-	m_renshaTurboSlider->SetTickFreq(5, 1);
+	m_speedSlider->SetTickFreq(25);
+	m_maxFrameSkipSlider->SetTickFreq(5);
+	m_fastForwardSpeedSlider->SetTickFreq(250);
+	m_renshaTurboSlider->SetTickFreq(5);
 
 	m_printerportFileLabel = (wxStaticText*)FindWindowByName(wxT("PrinterLogFileLabel"));
 	m_printerportLabel     = (wxStaticText*)FindWindowByName(wxT("PrinterLabel"));
 	m_renshaLabel          = (wxStaticText*)FindWindowByName(wxT("RenShaTurboLabel"));
-	m_frameskipLabel       = (wxStaticText*)FindWindowByName(wxT("FrameSkipLabel"));
 	m_frameskipMaxLabel    = (wxStaticText*)FindWindowByName(wxT("FrameSkipMaxLabel"));
-	m_frameskipMinLabel    = (wxStaticText*)FindWindowByName(wxT("FrameSkipMinLabel"));
+	m_fastForwardSpeedLabel= (wxStaticText*)FindWindowByName(wxT("FastForwardSpeedLabel"));
 	m_emulationSpeedLabel  = (wxStaticText*)FindWindowByName(wxT("EmulationSpeedLabel"));
 
 	auto* box = (wxComboBox*)FindWindowByName(wxT("PrinterportSelector"));
@@ -175,10 +175,10 @@ void MiscControlPage::OnFirmware(wxCommandEvent& event)
 
 void MiscControlPage::OnSpeedChange(wxScrollEvent& event)
 {
-	auto speedText = wxString::Format(wxT("%ld"), event.GetInt());
+	auto speedText = wxString::Format(wxT("%d"), event.GetInt());
 	m_speedIndicator->SetValue(speedText);
 	m_controller.WriteCommand(wxT("set speed ") + speedText);
-	m_controller.WriteCommand(wxT("set throttle on"));
+	m_controller.WriteCommand(wxT("set throttle on; set fastforward off"));
 }
 
 void MiscControlPage::OnSetNormalSpeed(wxCommandEvent& event)
@@ -186,36 +186,45 @@ void MiscControlPage::OnSetNormalSpeed(wxCommandEvent& event)
 	m_controller.WriteCommand(wxT("set speed 100"));
 	m_speedIndicator->SetValue(wxT("100"));
 	m_speedSlider->SetValue(100);
-	m_controller.WriteCommand(wxT("set throttle on"));
+	m_controller.WriteCommand(wxT("set throttle on; set fastforward off"));
 }
 
-void MiscControlPage::OnSetMaxSpeed(wxCommandEvent& event)
+void MiscControlPage::OnSetFastForward(wxCommandEvent& event)
 {
-	if (m_speedMaxButton->GetValue()) {
+	if (m_fastForwardButton->GetValue()) {
 		m_speedSlider->Disable();
 		m_speedIndicator->Disable();
 		m_speedNormalButton->Disable();
-		m_controller.WriteCommand(wxT("set throttle off"));
+		m_controller.WriteCommand(wxT("set fastforward on"));
 	} else {
 		m_speedSlider->Enable();
 		m_speedIndicator->Enable();
 		m_speedNormalButton->Enable();
-		m_controller.WriteCommand(wxT("set throttle on"));
+		m_controller.WriteCommand(wxT("set fastforward off"));
+	}
+}
+
+void MiscControlPage::OnSetFastLoading(wxCommandEvent& event)
+{
+	if (m_fastLoadingButton->GetValue()) {
+		m_controller.WriteCommand(wxT("set fullspeedwhenloading on"));
+	} else {
+		m_controller.WriteCommand(wxT("set fullspeedwhenloading off"));
 	}
 }
 
 void MiscControlPage::OnMaxFrameSkipChange(wxScrollEvent& event)
 {
-	auto skipText = wxString::Format(wxT("%ld"), event.GetInt());
+	auto skipText = wxString::Format(wxT("%d"), event.GetInt());
 	m_maxFrameSkipIndicator->SetValue(skipText);
 	m_controller.WriteCommand(wxT("set maxframeskip ") + skipText);
 }
 
-void MiscControlPage::OnMinFrameSkipChange(wxScrollEvent& event)
+void MiscControlPage::OnFastForwardSpeedChange(wxScrollEvent& event)
 {
-	auto skipText = wxString::Format(wxT("%ld"), event.GetInt());
-	m_minFrameSkipIndicator->SetValue(skipText);
-	m_controller.WriteCommand(wxT("set minframeskip ") + skipText);
+	auto skipText = wxString::Format(wxT("%d"), event.GetInt());
+	m_fastForwardSpeedIndicator->SetValue(skipText);
+	m_controller.WriteCommand(wxT("set fastforwardspeed ") + skipText);
 }
 
 void MiscControlPage::OnSetDefaultMaxFrameSkip(wxCommandEvent& event)
@@ -225,23 +234,11 @@ void MiscControlPage::OnSetDefaultMaxFrameSkip(wxCommandEvent& event)
 	m_maxFrameSkipSlider->Enable();
 }
 
-void MiscControlPage::OnSetDefaultMinFrameSkip(wxCommandEvent& event)
+void MiscControlPage::OnSetDefaultFastForwardSpeed(wxCommandEvent& event)
 {
-	m_controller.WriteCommand(wxT("unset minframeskip"));
-	m_minFrameSkipIndicator->Enable();
-	m_minFrameSkipSlider->Enable();
-}
-
-void MiscControlPage::OnSetThrottle(wxCommandEvent& event)
-{
-	auto* button = (wxToggleButton*)event.GetEventObject();
-	if (button->GetValue()) {
-		m_controller.WriteCommand(wxT("set throttle on"));
-		button->SetLabel(wxT("On"));
-	} else {
-		m_controller.WriteCommand(wxT("set throttle off"));
-		button->SetLabel(wxT("Off"));
-	}
+	m_controller.WriteCommand(wxT("unset fastforwardspeed"));
+	m_fastForwardSpeedIndicator->Enable();
+	m_fastForwardSpeedSlider->Enable();
 }
 
 void MiscControlPage::OnSetCmdTiming(wxCommandEvent& event)
@@ -261,28 +258,29 @@ void MiscControlPage::SetControlsOnLaunch()
 	m_speedSlider->Enable(true);
 	m_speedIndicator->Enable(true);
 	m_speedNormalButton->Enable(true);
-	m_speedMaxButton->Enable(true);
+	m_fastForwardButton->Enable(true);
+	m_fastLoadingButton->Enable(true);
+
+	m_fastForwardSpeedSlider->Enable(true);
+	m_fastForwardSpeedIndicator->Enable(true);
+	m_defaultFastForwardSpeedButton->Enable(true);
 
 	m_maxFrameSkipSlider->Enable(true);
-	m_minFrameSkipSlider->Enable(true);
-	m_minFrameSkipIndicator->Enable(true);
 	m_maxFrameSkipIndicator->Enable(true);
-	m_defaultMinFrameSkipButton->Enable(true);
 	m_defaultMaxFrameSkipButton->Enable(true);
 
 	m_powerButton->Enable(true);
 	m_resetButton->Enable(true);
 	m_pauseButton->Enable(true);
-	m_printerportSelector->Enable(true);
 	m_pauseButton->SetValue(false);
 	m_firmwareButton->SetValue(false);
+	m_printerportSelector->Enable(true);
 	m_printerLogFile->Enable(true);
 	m_browsePrinterLog->Enable(true);
 	m_printerportFileLabel->Enable(true);
 	m_printerportLabel->Enable(true);
-	m_frameskipLabel->Enable(true);
 	m_frameskipMaxLabel->Enable(true);
-	m_frameskipMinLabel->Enable(true);
+	m_fastForwardSpeedLabel->Enable(true);
 	m_emulationSpeedLabel->Enable(true);
 	if (auto* temp = FindWindowByLabel(wxT("Performance Controls"))) {
 		temp->Enable(true);
@@ -309,13 +307,15 @@ void MiscControlPage::SetControlsOnEnd()
 	m_speedSlider->Enable(false);
 	m_speedIndicator->Enable(false);
 	m_speedNormalButton->Enable(false);
-	m_speedMaxButton->Enable(false);
+	m_fastForwardButton->Enable(false);
+	m_fastLoadingButton->Enable(false);
+
+	m_fastForwardSpeedSlider->Enable(false);
+	m_fastForwardSpeedIndicator->Enable(false);
+	m_defaultFastForwardSpeedButton->Enable(false);
 
 	m_maxFrameSkipSlider->Enable(false);
-	m_minFrameSkipSlider->Enable(false);
-	m_minFrameSkipIndicator->Enable(false);
 	m_maxFrameSkipIndicator->Enable(false);
-	m_defaultMinFrameSkipButton->Enable(false);
 	m_defaultMaxFrameSkipButton->Enable(false);
 
 	m_powerButton->Enable(false);
@@ -329,9 +329,8 @@ void MiscControlPage::SetControlsOnEnd()
 	m_printerportFileLabel->Enable(false);
 	m_printerportLabel->Enable(false);
 	m_renshaLabel->Enable(false);
-	m_frameskipLabel->Enable(false);
 	m_frameskipMaxLabel->Enable(false);
-	m_frameskipMinLabel->Enable(false);
+	m_fastForwardSpeedLabel->Enable(false);
 	m_emulationSpeedLabel->Enable(false);
 	if (auto* temp = FindWindowByLabel(wxT("Performance Controls"))) {
 		temp->Enable(false);
@@ -359,19 +358,19 @@ void MiscControlPage::OnInputSpeed(wxCommandEvent& event)
 	}
 }
 
-void MiscControlPage::OnInputMinFrameskip(wxCommandEvent& event)
+void MiscControlPage::OnInputFastForwardSpeed(wxCommandEvent& event)
 {
-	wxString text = m_minFrameSkipIndicator->GetValue();
+	wxString text = m_fastForwardSpeedIndicator->GetValue();
 	if (text.IsNumber()) {
 		unsigned long num;
 		text.ToULong(&num);
-		if (num > 100) {
-			num = 100;
-			text = wxT("100");
-			m_minFrameSkipIndicator->SetValue(text);
+		if (num > 10000) {
+			num = 10000;
+			text = wxT("10000");
+			m_fastForwardSpeedIndicator->SetValue(text);
 		}
-		m_controller.WriteCommand(wxT("set minframeskip ") + text);
-		m_minFrameSkipSlider->SetValue(num);
+		m_controller.WriteCommand(wxT("set fastforwardspeed ") + text);
+		m_fastForwardSpeedSlider->SetValue(num);
 	}
 }
 
@@ -393,26 +392,29 @@ void MiscControlPage::OnInputMaxFrameskip(wxCommandEvent& event)
 
 void MiscControlPage::InitConnectorPanel()
 {
+	// first disable them all, for cases where we don't have that whole connector present
+	FindWindowByName(wxT("PrinterportSelector"))->Enable(false);
+	FindWindowByName(wxT("Joyport1Selector"))->Enable(false);
+	FindWindowByName(wxT("Joyport2Selector"))->Enable(false);
 	for (auto& connector : m_controller.GetConnectors()) {
 		wxString currentClass = m_controller.GetConnectorClass(connector);
 		if (connector == wxT("joyporta")) {
-			InitJoystickPort(connector, wxT("Joyport1Selector"), currentClass);
+			InitJoystickOrPrinterPort(connector, wxT("Joyport1Selector"), currentClass);
 		} else if (connector == wxT("joyportb")) {
-			InitJoystickPort(connector, wxT("Joyport2Selector"), currentClass);
-//		} else if (connector] == wxT("printerport")) {
-//			InitJoystickPort(connector, wxT("PrinterportSelector"), currentClass);
+			InitJoystickOrPrinterPort(connector, wxT("Joyport2Selector"), currentClass);
+		} else if (connector == wxT("printerport")) {
+			InitJoystickOrPrinterPort(connector, wxT("PrinterportSelector"), currentClass);
 		}
 	}
-//	auto* text = (wxTextCtrl*)FindWindowByName("PrinterLogFile");
-//	m_controller.WriteCommand(wxString("set printerlogfilename ") + utils::ConvertPath(text->GetValue()));
 }
 
-void MiscControlPage::InitJoystickPort(wxString connector, wxString control, wxString connectorClass)
+void MiscControlPage::InitJoystickOrPrinterPort(wxString connector, wxString control, wxString connectorClass)
 {
 	const wxArrayString& classes = m_controller.GetPluggableClasses();
 	if (classes.IsEmpty()) return;
 
 	auto* box = (wxComboBox*)FindWindowByName(control);
+	box->Enable(true);
 	wxString currentval = box->GetValue();
 	box->Clear();
 	box->Append(wxT("--empty--"));
@@ -423,16 +425,22 @@ void MiscControlPage::InitJoystickPort(wxString connector, wxString control, wxS
 			box->Append(pluggables[i]);
 		}
 	}
-	if (box->FindString(currentval) == wxNOT_FOUND) {
-		currentval = box->GetString(0);
+	if (m_controller.IsRelaunching()) {
+		// if relaunching, then just update the values
+		wxString currentlyPlugged = m_controller.GetConnectorPlugged(connector);
+		box->SetValue(currentlyPlugged.IsEmpty() ? wxT("--empty--") : currentlyPlugged);
+	} else {
+		// Initializing (initial launch)
+		if (box->FindString(currentval) == wxNOT_FOUND) {
+			currentval = box->GetString(0);
+		}
+		box->SetValue(currentval);
+		if (currentval != wxT("--empty--")) {
+			m_controller.WriteCommand(wxT("plug ") + connector + wxT(" ") + currentval);
+		} else {
+			m_controller.WriteCommand(wxT("unplug ") + connector);
+		}
 	}
-	box->SetValue(currentval);
-	wxString cmd = wxT("plug");
-	if (currentval == wxT("--empty--")) {
-		cmd = wxT("unplug");
-		currentval.Clear();
-	}
-	m_controller.WriteCommand(cmd + wxT(" ") + connector + wxT(" ") + currentval);
 }
 
 void MiscControlPage::OnChangeJoystick(wxCommandEvent& event)
@@ -485,23 +493,23 @@ void MiscControlPage::OnJoystickChanged()
 		cmd = wxT("plug ");
 	}
 
-	bool flag1=false;
-	if (box->GetValue() != wxT("--empty--")){
-		assert(box2!=0);
-		if(!box2)throw "box2 must not be null, but it is null.";
+	bool sameDevicePluggedInTwoPorts = false;
+	if (box->GetValue() != wxT("--empty--")) {
+		assert(box2 != 0);
+		if (!box2) throw "box2 must not be null, but it is null.";
 
-		if(box->GetValue() == box2->GetValue()){
-			flag1=true;
-			assert(oldValue1!=0);
-			if(!oldValue1)throw "oldValue1 must not be null, but it is null.";
+		if (box->GetValue() == box2->GetValue()) {
+			sameDevicePluggedInTwoPorts = true;
+			assert(oldValue1 != 0);
+			if (!oldValue1) throw "oldValue1 must not be null, but it is null.";
 			MessageActive = true;
 			int result = wxMessageBox(wxT("Unable to plug a device in more than one port!\n\nDo you still want to plug it into this port?\nThis device will then be removed from any other port(s)."),
 				wxT("Warning"),
 				wxOK | wxCANCEL);
 			MessageActive = false;
 			if (result == wxOK) {
-				assert(oldValue2!=0);
-				if(!oldValue2)throw "oldValue2 must not be null, but it is null.";
+				assert(oldValue2 != 0);
+				if (!oldValue2) throw "oldValue2 must not be null, but it is null.";
 
 				box2->SetSelection(0);
 				*oldValue2 = wxT("--empty--");
@@ -514,7 +522,7 @@ void MiscControlPage::OnJoystickChanged()
 			}
 		}
 	}
-	if(!flag1){
+	if (!sameDevicePluggedInTwoPorts) {
 		// no collision
 		m_controller.WriteCommand(cmd + connector + value);
 		assert(oldValue1!=0);
@@ -526,13 +534,12 @@ void MiscControlPage::OnJoystickChanged()
 	ConfigurationData::instance().SetParameter(ConfigurationData::CD_JOYPORT1, joy1->GetValue());
 	ConfigurationData::instance().SetParameter(ConfigurationData::CD_JOYPORT2, joy2->GetValue());
 	ConfigurationData::instance().SaveData();
-
 }
 
 void MiscControlPage::OnChangeRenShaTurbo(wxScrollEvent& event)
 {
 	m_controller.WriteCommand(wxString::Format(
-		wxT("set renshaturbo %ld"), event.GetInt()));
+		wxT("set renshaturbo %d"), event.GetInt()));
 }
 
 void MiscControlPage::OnChangePrinterPort(wxCommandEvent& event)
@@ -567,7 +574,7 @@ void MiscControlPage::OnChangePrinterLogFile(wxCommandEvent& event)
 	wxString current = ((wxTextCtrl*)event.GetEventObject())->GetValue();
 	ConfigurationData::instance().SetParameter(ConfigurationData::CD_PRINTERFILE, current);
 	ConfigurationData::instance().SaveData();
-	m_controller.WriteCommand(wxT("set printerlogfilename ") + utils::ConvertPath(current));
+	m_controller.WriteCommand(wxT("set printerlogfilename ") + utils::ConvertPathForCommand(current));
 }
 
 void MiscControlPage::OnBrowsePrinterLogFile(wxCommandEvent& event)
