@@ -17,16 +17,6 @@ class wxStaticText;
 class wxToggleButton;
 class wxListBox;
 
-class SessionDropTarget : public wxFileDropTarget
-{
-public:
-	SessionDropTarget(wxComboBox* target);
-	bool OnDropFiles(wxCoord x, wxCoord y, const wxArrayString& filenames);
-
-private:
-	wxComboBox* m_target;
-};
-
 
 class SessionPage : public CatapultPage
 {
@@ -48,10 +38,13 @@ public:
 	void SetCassetteMode(wxString data);
 	const wxArrayString& GetDetectedMachines() const;
 	const wxArrayString& GetDetectedExtensions() const;
+	void GetRomTypes();
 	void RestoreHistory();
 	void FixLayout();
 	wxString getStartupCommandLineOptions() const;
 
+	friend class SessionDropTarget;
+
 private:
 	enum MediaType { DISKETTE, CARTRIDGE, CASSETTE, HARDDISK };
 	struct MediaInfo {
@@ -136,15 +129,14 @@ private:
 
 	void SetupRomType(wxString romtype, wxString fullname);
 	MediaInfo* GetLastMenuTarget() const;
-	void GetRomTypes();
 	void SetIpsList(MediaInfo& m, const wxArrayString& ips);
 	void SetMapperType(MediaInfo& media, const wxString& type);
 	void BrowseDisk(MediaInfo& media);
 	void BrowseCart(MediaInfo& media);
 	void BrowseMedia(MediaInfo& m, const wxString& path, const wxString title);
 	void ClickMediaCombo(wxCommandEvent& event, MediaInfo& media);
-	void prepareMachines  (const wxString& sharepath, bool optional = false);
-	void prepareExtensions(const wxString& sharepath, bool optional = false);
+	void prepareMachines  ();
+	void prepareExtensions();
 	void AddHistory(MediaInfo& media);
 	void SaveHistory();
 	void EjectMedia(MediaInfo& media);
@@ -196,4 +188,16 @@ private:
 	DECLARE_EVENT_TABLE()
 };
 
+class SessionDropTarget : public wxFileDropTarget
+{
+public:
+	SessionDropTarget(SessionPage::MediaInfo* target, SessionPage* sessionPage);
+	bool OnDropFiles(wxCoord x, wxCoord y, const wxArrayString& filenames);
+
+private:
+	SessionPage::MediaInfo* m_target;
+	SessionPage* m_sessionPage;
+};
+
+
 #endif
