@@ -13,19 +13,19 @@
 #include <wx/textctrl.h>
 #include <wx/wxprec.h>
 #include <wx/xrc/xmlres.h>
+#include <wx/filefn.h>
 
 IMPLEMENT_CLASS(VideoControlPage, wxPanel)
 BEGIN_EVENT_TABLE(VideoControlPage, wxPanel)
 	EVT_COMBOBOX(      XRCID("RendererSelector"),       CatapultPage::OnClickCombo)
 	EVT_COMBOBOX(      XRCID("ScalerAlgoSelector"),     CatapultPage::OnClickCombo)
 	EVT_COMBOBOX(      XRCID("ScalerFactorSelector"),   CatapultPage::OnClickCombo)
-	EVT_COMBOBOX(      XRCID("AccuracySelector"),       CatapultPage::OnClickCombo)
 	EVT_COMBOBOX(      XRCID("VideoSourceSelector"),    CatapultPage::OnClickCombo)
 	EVT_TEXT(          XRCID("RendererSelector"),       VideoControlPage::OnChangeRenderer)
 	EVT_TEXT(          XRCID("ScalerAlgoSelector"),     VideoControlPage::OnChangeScalerAlgo)
 	EVT_TEXT(          XRCID("ScalerFactorSelector"),   VideoControlPage::OnChangeScalerFactor)
-	EVT_TEXT(          XRCID("AccuracySelector"),       VideoControlPage::OnChangeAccuracy)
 	EVT_TEXT(          XRCID("VideoSourceSelector"),    VideoControlPage::OnChangeVideoSource)
+	EVT_TOGGLEBUTTON(  XRCID("VSyncButton"),            VideoControlPage::OnVSync)
 	EVT_TOGGLEBUTTON(  XRCID("DeInterlaceButton"),      VideoControlPage::OnDeInterlace)
 	EVT_TOGGLEBUTTON(  XRCID("LimitSpriteButton"),      VideoControlPage::OnLimitSprites)
 	EVT_TOGGLEBUTTON(  XRCID("DisableSpritesButton"),   VideoControlPage::OnDisableSprites)
@@ -33,12 +33,15 @@ BEGIN_EVENT_TABLE(VideoControlPage, wxPanel)
 	EVT_COMMAND_SCROLL(XRCID("BlurSlider"),             VideoControlPage::OnChangeBlur)
 	EVT_COMMAND_SCROLL(XRCID("GlowSlider"),             VideoControlPage::OnChangeGlow)
 	EVT_COMMAND_SCROLL(XRCID("GammaSlider"),            VideoControlPage::OnChangeGamma)
+	EVT_COMMAND_SCROLL(XRCID("NoiseSlider"),            VideoControlPage::OnChangeNoise)
 	EVT_COMMAND_SCROLL(XRCID("ScanLineSlider"),         VideoControlPage::OnChangeScanlines)
 	EVT_BUTTON(        XRCID("ZeroBlurButton"),         VideoControlPage::OnDefaultBlur)
 	EVT_BUTTON(        XRCID("ZeroGlowButton"),         VideoControlPage::OnDefaultGlow)
 	EVT_BUTTON(        XRCID("DefaultGammaButton"),     VideoControlPage::OnDefaultGamma)
+	EVT_BUTTON(        XRCID("DefaultNoiseButton"),     VideoControlPage::OnDefaultNoise)
 	EVT_BUTTON(        XRCID("ZeroScanlineButton"),     VideoControlPage::OnDefaultScanlines)
 	EVT_TEXT(          XRCID("GammaIndicator"),         VideoControlPage::OnInputGamma)
+	EVT_TEXT(          XRCID("NoiseIndicator"),         VideoControlPage::OnInputNoise)
 	EVT_TEXT(          XRCID("BlurIndicator"),          VideoControlPage::OnInputBlur)
 	EVT_TEXT(          XRCID("GlowIndicator"),          VideoControlPage::OnInputGlow)
 	EVT_TEXT(          XRCID("ScanlineIndicator"),      VideoControlPage::OnInputScanline)
@@ -51,31 +54,35 @@ VideoControlPage::VideoControlPage(wxWindow* parent, openMSXController& controll
 	: m_controller(controller)
 {
 	wxXmlResource::Get()->LoadPanel(this, parent, wxT("VideoControlPage"));
-	m_rendererList     = (wxComboBox*)FindWindowByName(wxT("RendererSelector"));;
+	m_rendererList     = (wxComboBox*)FindWindowByName(wxT("RendererSelector"));
 	m_scalerAlgoList   = (wxComboBox*)FindWindowByName(wxT("ScalerAlgoSelector"));
-	m_timesLabel     = (wxStaticText*)FindWindowByName(wxT("times"));
-	m_scalerFactorList = (wxComboBox*)FindWindowByName(wxT("ScalerFactorSelector"));
-	m_accuracyList     = (wxComboBox*)FindWindowByName(wxT("AccuracySelector"));
+	m_timesLabel       = (wxStaticText*)FindWindowByName(wxT("times"));
+	m_scalerFactorList   = (wxComboBox*)FindWindowByName(wxT("ScalerFactorSelector"));
+	m_vSyncButton        = (wxToggleButton*)FindWindowByName(wxT("VSyncButton"));
 	m_videoSourceList    = (wxComboBox*)FindWindowByName(wxT("VideoSourceSelector"));
 	m_deinterlaceButton  = (wxToggleButton*)FindWindowByName(wxT("DeInterlaceButton"));
 	m_limitSpritesButton = (wxToggleButton*)FindWindowByName(wxT("LimitSpriteButton"));
 	m_disableSpritesButton = (wxToggleButton*)FindWindowByName(wxT("DisableSpritesButton"));
 	m_fullscreenButton   = (wxToggleButton*)FindWindowByName(wxT("FullScreenButton"));
 	m_blurSlider     = (wxSlider*)FindWindowByName(wxT("BlurSlider"));
-	m_blurSlider->SetTickFreq(5, 1);
+	m_blurSlider->SetTickFreq(5);
 	m_glowSlider     = (wxSlider*)FindWindowByName(wxT("GlowSlider"));
-	m_glowSlider->SetTickFreq(5, 1);
+	m_glowSlider->SetTickFreq(5);
 	m_gammaSlider    = (wxSlider*)FindWindowByName(wxT("GammaSlider"));
-	m_gammaSlider->SetTickFreq(25, 1);
+	m_gammaSlider->SetTickFreq(25);
+	m_noiseSlider    = (wxSlider*)FindWindowByName(wxT("NoiseSlider"));
+	m_noiseSlider->SetTickFreq(25);
 	m_scanlineSlider = (wxSlider*)FindWindowByName(wxT("ScanLineSlider"));
-	m_scanlineSlider->SetTickFreq(5, 1);
+	m_scanlineSlider->SetTickFreq(5);
 	m_blurIndicator     = (wxTextCtrl*)FindWindowByName(wxT("BlurIndicator"));
 	m_glowIndicator     = (wxTextCtrl*)FindWindowByName(wxT("GlowIndicator"));
 	m_gammaIndicator    = (wxTextCtrl*)FindWindowByName(wxT("GammaIndicator"));
+	m_noiseIndicator    = (wxTextCtrl*)FindWindowByName(wxT("NoiseIndicator"));
 	m_scanlineIndicator = (wxTextCtrl*)FindWindowByName(wxT("ScanlineIndicator"));
 	m_defaultBlurButton = (wxButton*)FindWindowByName(wxT("ZeroBlurButton"));
 	m_defaultGlowButton     = (wxButton*)FindWindowByName(wxT("ZeroGlowButton"));
 	m_defaultGammaButton    = (wxButton*)FindWindowByName(wxT("DefaultGammaButton"));
+	m_defaultNoiseButton    = (wxButton*)FindWindowByName(wxT("DefaultNoiseButton"));
 	m_defaultScanlineButton = (wxButton*)FindWindowByName(wxT("ZeroScanlineButton"));
 	m_screenShotFile    = (wxTextCtrl*)FindWindowByName(wxT("ScreenShotFilename"));
 	m_screenShotCounter = (wxTextCtrl*)FindWindowByName(wxT("ScreenShotCounter"));
@@ -83,7 +90,7 @@ VideoControlPage::VideoControlPage(wxWindow* parent, openMSXController& controll
 
 	m_rendererLabel      = (wxStaticText*)FindWindowByName(wxT("RendererLabel"));
 	m_scalerLabel        = (wxStaticText*)FindWindowByName(wxT("ScalerLabel"));
-	m_accuracyLabel      = (wxStaticText*)FindWindowByName(wxT("AccuracyLabel"));
+	m_vSyncLabel         = (wxStaticText*)FindWindowByName(wxT("VSyncLabel"));
 	m_videoSourceLabel   = (wxStaticText*)FindWindowByName(wxT("VideoSourceLabel"));
 	m_deinterlaceLabel   = (wxStaticText*)FindWindowByName(wxT("DeInterlaceLabel"));
 	m_limitSpritesLabel  = (wxStaticText*)FindWindowByName(wxT("LimitSpriteLabel"));
@@ -93,8 +100,11 @@ VideoControlPage::VideoControlPage(wxWindow* parent, openMSXController& controll
 	m_blurLabel          = (wxStaticText*)FindWindowByName(wxT("BlurLabel"));
 	m_glowLabel          = (wxStaticText*)FindWindowByName(wxT("GlowLabel"));
 	m_gammaLabel         = (wxStaticText*)FindWindowByName(wxT("GammaLabel"));
+	m_noiseLabel         = (wxStaticText*)FindWindowByName(wxT("NoiseLabel"));
 
 	setNewRenderersAndScalers();
+	wxCommandEvent event;
+	OnChangeScreenshotFilename(event);
 }
 
 void VideoControlPage::OnChangeRenderer(wxCommandEvent& event)
@@ -112,9 +122,16 @@ void VideoControlPage::OnChangeScalerFactor(wxCommandEvent& event)
 	m_controller.WriteCommand(wxT("set scale_factor ") + m_scalerFactorList->GetValue().Lower());
 }
 
-void VideoControlPage::OnChangeAccuracy(wxCommandEvent& event)
+void VideoControlPage::OnVSync(wxCommandEvent& event)
 {
-	m_controller.WriteCommand(wxT("set accuracy ") + m_accuracyList->GetValue().Lower());
+	auto* button = (wxToggleButton*)event.GetEventObject();
+	if (button->GetValue()) {
+		m_controller.WriteCommand(wxT("set vsync on"));
+		button->SetLabel(wxT("On"));
+	} else {
+		m_controller.WriteCommand(wxT("set vsync off"));
+		button->SetLabel(wxT("Off"));
+	}
 }
 
 void VideoControlPage::OnChangeVideoSource(wxCommandEvent& event)
@@ -196,14 +213,14 @@ void VideoControlPage::OnFullScreen(wxCommandEvent& event)
 
 void VideoControlPage::OnChangeBlur(wxScrollEvent& event)
 {
-	auto text = wxString::Format(wxT("%ld"), event.GetInt());
+	auto text = wxString::Format(wxT("%d"), event.GetInt());
 	m_blurIndicator->SetValue(text);
 	m_controller.WriteCommand(wxT("set blur ") + text);
 }
 
 void VideoControlPage::OnChangeGlow(wxScrollEvent& event)
 {
-	auto text = wxString::Format(wxT("%ld"), event.GetInt());
+	auto text = wxString::Format(wxT("%d"), event.GetInt());
 	m_glowIndicator->SetValue(text);
 	m_controller.WriteCommand(wxT("set glow ") + text);
 }
@@ -215,9 +232,16 @@ void VideoControlPage::OnChangeGamma(wxScrollEvent& event)
 	m_controller.WriteCommand(wxT("set gamma ") + text);
 }
 
+void VideoControlPage::OnChangeNoise(wxScrollEvent& event)
+{
+	auto text = wxString::Format(wxT("%.1f"), event.GetInt() / 10.0);
+	m_noiseIndicator->SetValue(text);
+	m_controller.WriteCommand(wxT("set noise ") + text);
+}
+
 void VideoControlPage::OnChangeScanlines(wxScrollEvent& event)
 {
-	auto text = wxString::Format(wxT("%ld"), event.GetInt());
+	auto text = wxString::Format(wxT("%d"), event.GetInt());
 	m_scanlineIndicator->SetValue(text);
 	m_controller.WriteCommand(wxT("set scanline ") + text);
 }
@@ -237,6 +261,11 @@ void VideoControlPage::OnDefaultGamma(wxCommandEvent& event)
 	m_gammaIndicator->SetValue(m_defaultGamma);
 }
 
+void VideoControlPage::OnDefaultNoise(wxCommandEvent& event)
+{
+	m_noiseIndicator->SetValue(m_defaultNoise);
+}
+
 void VideoControlPage::OnDefaultScanlines(wxCommandEvent& event)
 {
 	m_scanlineIndicator->SetValue(m_defaultScanline);
@@ -293,6 +322,23 @@ void VideoControlPage::OnInputGamma(wxCommandEvent& event)
 	}
 }
 
+void VideoControlPage::OnInputNoise(wxCommandEvent& event)
+{
+	wxString text = m_noiseIndicator->GetValue();
+	double floatNumber;
+	unsigned long num = 0;
+	if (text.ToDouble(&floatNumber) && (text[text.Len() - 1] != '.')) {
+		num = (int)(floatNumber * 10);
+		if (num > 1000) {
+			num = 1000;
+			text = wxT("100.0");
+			m_noiseIndicator->SetValue(text);
+		}
+		m_controller.WriteCommand(wxT("set noise ") + text);
+		m_noiseSlider->SetValue(num);
+	}
+}
+
 void VideoControlPage::OnInputScanline(wxCommandEvent& event)
 {
 	wxString text = m_scanlineIndicator->GetValue();
@@ -320,6 +366,9 @@ void VideoControlPage::SetControlsOnLaunch()
 	m_gammaSlider->Enable(true);
 	m_gammaIndicator->Enable(true);
 	m_defaultGammaButton->Enable(true);
+	m_noiseSlider->Enable(true);
+	m_noiseIndicator->Enable(true);
+	m_defaultNoiseButton->Enable(true);
 	m_scanlineSlider->Enable(true);
 	m_scanlineIndicator->Enable(true);
 	m_defaultScanlineButton->Enable(true);
@@ -327,7 +376,7 @@ void VideoControlPage::SetControlsOnLaunch()
 	m_scalerAlgoList->Enable(true);
 	m_timesLabel->Enable(true);
 	m_scalerFactorList->Enable(true);
-	m_accuracyList->Enable(true);
+	m_vSyncButton->Enable(true);
 	m_videoSourceList->Enable(true);
 	m_deinterlaceButton->Enable(true);
 	m_limitSpritesButton->Enable(true);
@@ -336,7 +385,7 @@ void VideoControlPage::SetControlsOnLaunch()
 	m_screenShotButton->Enable(true);
 	m_rendererLabel->Enable(true);
 	m_scalerLabel->Enable(true);
-	m_accuracyLabel->Enable(true);
+	m_vSyncLabel->Enable(true);
 	m_videoSourceLabel->Enable(true);
 	m_deinterlaceLabel->Enable(true);
 	m_limitSpritesLabel->Enable(true);
@@ -346,6 +395,7 @@ void VideoControlPage::SetControlsOnLaunch()
 	m_blurLabel->Enable(true);
 	m_glowLabel->Enable(true);
 	m_gammaLabel->Enable(true);
+	m_noiseLabel->Enable(true);
 	if (auto* temp = FindWindowByLabel(wxT("Switches"))) {
 		temp->Enable(true);
 	}
@@ -365,6 +415,9 @@ void VideoControlPage::SetControlsOnEnd()
 	m_gammaSlider->Enable(false);
 	m_gammaIndicator->Enable(false);
 	m_defaultGammaButton->Enable(false);
+	m_noiseSlider->Enable(false);
+	m_noiseIndicator->Enable(false);
+	m_defaultNoiseButton->Enable(false);
 	m_scanlineSlider->Enable(false);
 	m_scanlineIndicator->Enable(false);
 	m_defaultScanlineButton->Enable(false);
@@ -372,7 +425,7 @@ void VideoControlPage::SetControlsOnEnd()
 	m_scalerAlgoList->Enable(false);
 	m_timesLabel->Enable(false);
 	m_scalerFactorList->Enable(false);
-	m_accuracyList->Enable(false);
+	m_vSyncButton->Enable(false);
 	m_videoSourceList->Enable(false);
 	m_deinterlaceButton->Enable(false);
 	m_limitSpritesButton->Enable(false);
@@ -381,7 +434,7 @@ void VideoControlPage::SetControlsOnEnd()
 	m_screenShotButton->Enable(false);
 	m_rendererLabel->Enable(false);
 	m_scalerLabel->Enable(false);
-	m_accuracyLabel->Enable(false);
+	m_vSyncLabel->Enable(false);
 	m_videoSourceLabel->Enable(false);
 	m_deinterlaceLabel->Enable(false);
 	m_limitSpritesLabel->Enable(false);
@@ -391,6 +444,7 @@ void VideoControlPage::SetControlsOnEnd()
 	m_blurLabel->Enable(false);
 	m_glowLabel->Enable(false);
 	m_gammaLabel->Enable(false);
+	m_noiseLabel->Enable(false);
 	if (auto* temp = FindWindowByLabel(wxT("Switches"))) {
 		temp->Enable(false);
 	}
@@ -404,6 +458,7 @@ void VideoControlPage::SetSliderDefaults()
 	m_defaultBlur = m_blurIndicator->GetValue();
 	m_defaultGlow = m_glowIndicator->GetValue();
 	m_defaultGamma = m_gammaIndicator->GetValue();
+	m_defaultNoise = m_noiseIndicator->GetValue();
 	m_defaultScanline = m_scanlineIndicator->GetValue();
 }
 
@@ -435,8 +490,12 @@ void VideoControlPage::OnTakeScreenShot(wxCommandEvent& event)
 	wxString screenshotfile = m_screenShotFile->GetValue();
 	wxString counter = m_screenShotCounter->GetValue();
 	wxString cmd = wxT("screenshot");
-	if (!screenshotfile.IsEmpty() || !counter.IsEmpty()) {
-		cmd << wxT(" ") << utils::ConvertPath(screenshotfile) << counter << wxT(".png");
+	if (!screenshotfile.IsEmpty()) {
+		if (wxIsAbsolutePath(screenshotfile)) {
+			cmd << wxT(" ") << utils::ConvertPathForCommand(screenshotfile + counter + wxT(".png"));
+		} else {
+			cmd << wxT(" -prefix ") << screenshotfile;
+		}
 	}
 	m_controller.WriteCommand(cmd,
 		[&](const wxString&, const wxString&) { UpdateScreenshotCounter(); });
@@ -457,17 +516,23 @@ static int FindFirstFreeScreenshotFile(wxString prefix)
 
 void VideoControlPage::OnChangeScreenshotFilename(wxCommandEvent& event)
 {
+	m_screenShotCounter->Enable(false);
 	if (m_screenShotFile->GetValue().IsEmpty()) {
 		m_screenShotCounter->Clear();
 	} else {
-		UpdateScreenshotCounter();
+		if (wxIsAbsolutePath(m_screenShotFile->GetValue())) {
+			m_screenShotCounter->Enable(true);
+			UpdateScreenshotCounter();
+		} else {
+			m_screenShotCounter->Clear();
+		}
 	}
 }
 
 void VideoControlPage::UpdateScreenshotCounter()
 {
 	wxString prefix = m_screenShotFile->GetValue();
-	if (!prefix.IsEmpty()) {
+	if (!prefix.IsEmpty() && wxIsAbsolutePath(prefix)) {
 		auto count = wxString::Format(
 			wxT("%04d"), FindFirstFreeScreenshotFile(prefix));
 		m_screenShotCounter->SetValue(count);
@@ -479,6 +544,5 @@ void VideoControlPage::setNewRenderersAndScalers()
 	m_rendererList->Clear();
 	m_scalerAlgoList->Clear();
 	m_scalerFactorList->Clear();
-	m_accuracyList->Clear();
 	m_videoSourceList->Clear();
 }
