@@ -44,6 +44,7 @@ openMSXController::openMSXController(wxWindow* target)
 	m_openMSXID = 0;
 	m_appWindow = (wxCatapultFrame*)target;
 	m_openMsxRunning = false;
+	m_isRelaunching = false;
 	InitLaunchScript();
 }
 
@@ -156,14 +157,14 @@ void openMSXController::HandleParsedOutput(wxCommandEvent& event)
 			if ((lastCmd[0] != wxT("plug")) ||
 			    (lastCmd[1] != data->name)) {
 				UpdateSetting2(data->name, data->contents);
-				ExecuteStart(m_relaunch); // why?
-			}
-		} else if (data->updateType == CatapultXMLParser::UPDATE_UNPLUG) {
-			if ((lastCmd[0] != wxT("unplug")) ||
-			    (lastCmd[1] != data->name)) {
-				UpdateSetting2(data->name, data->contents);
-				ExecuteStart(m_relaunch); // why?
+				ExecuteStart(m_relaunch); // reinit stuff now
 			}
+		} else if (data->updateType == CatapultXMLParser::UPDATE_HARDWARE) {
+			ExecuteStart(m_relaunch); // reinit stuff now
+		} else if (data->updateType == CatapultXMLParser::UPDATE_EXTENSION) {
+			ExecuteStart(m_relaunch); // reinit stuff now
+		} else if (data->updateType == CatapultXMLParser::UPDATE_CONNECTOR) {
+			ExecuteStart(m_relaunch); // reinit stuff now
 		} else if (data->updateType == CatapultXMLParser::UPDATE_MEDIA) {
 			if ((lastCmd[0] != data->name) ||
 			    (lastCmd[1] != data->contents)) {
@@ -265,9 +266,9 @@ bool openMSXController::CheckVersion(const wxString& cmd)
 		? output[0] : wxString();
 
 	long ver = -1;
-	long majver;
-	long minver;
-	long subver;
+	long majver = 0;
+	long minver = 0;
+	long subver = 0;
 	if (version.StartsWith(wxT("openMSX "))) {
 		int pos = version.Find('.');
 		if (pos != wxNOT_FOUND) {
@@ -277,19 +278,19 @@ bool openMSXController::CheckVersion(const wxString& cmd)
 			if (pos != wxNOT_FOUND) {
 				version.Mid(0, pos).ToLong(&minver);
 				version.Mid(pos + 1).ToLong(&subver);
-				ver = (((majver * 100) + minver) * 100) + subver;
 			}
 		}
+		ver = (((majver * 100) + minver) * 100) + subver;
 	}
-	if (ver == -1) {
+	if (ver <= 0) {
 		wxMessageBox(
-			wxT("Unable to determine openMSX version!\nPlease upgrade to 0.6.2 or higher.\n(Or contact the authors.)"),
+			wxT("Unable to determine openMSX version!\nPlease upgrade to 0.10.0 or higher.\n(Or contact the authors.)"),
 			wxT("Error"));
 		return false;
 	}
-	if (ver < 602) {
+	if (ver < 100) {
 		wxMessageBox(
-			wxT("The openMSX version you are using is too old!\nPlease upgrade to 0.6.2 or higher."),
+			wxT("The openMSX version you are using is too old!\nPlease upgrade to 0.10.0 or higher."),
 			wxT("Error"));
 		return false;
 	}
@@ -314,6 +315,18 @@ wxString openMSXController::GetConnectorClass(const wxString& name) const
 	assert(false); return wxString();
 }
 
+wxString openMSXController::GetConnectorPlugged(const wxString& name) const
+{
+	if (m_connectorpluggeds.IsEmpty()) return wxString();
+
+	for (unsigned i = 0; i < m_connectors.GetCount(); ++i) {
+		if (m_connectors[i] == name) {
+			return m_connectorpluggeds[i];
+		}
+	}
+	assert(false); return wxString();
+}
+
 static bool isTclTrue(const wxString& str)
 {
 	return (str == wxT("on"))   ||
@@ -342,9 +355,6 @@ void openMSXController::InitLaunchScript()
 	AddCommand(wxT("lindex [openmsx_info setting scale_factor] 2"),
 		[&](const wxString&, const wxString& r) {
 			FillRangeComboBox(wxT("ScalerFactorSelector"), r); });
-	AddCommand(wxT("lindex [openmsx_info setting accuracy] 2"),
-		[&](const wxString&, const wxString& r) {
-			FillComboBox(wxT("AccuracySelector"), r); });
 	AddCommand(wxT("lindex [openmsx_info setting videosource] 2"),
 		[&](const wxString&, const wxString& r) {
 			FillComboBox(wxT("VideoSourceSelector"), r); });
@@ -364,7 +374,7 @@ void openMSXController::InitLaunchScript()
 	AddCommand(wxT("set scale_factor"),
 		[&](const wxString& c, const wxString& r) {
 			UpdateSetting(c, r); });
-	AddCommand(wxT("set accuracy"),
+	AddCommand(wxT("set vsync"),
 		[&](const wxString& c, const wxString& r) {
 			UpdateSetting(c, r); });
 	AddCommand(wxT("set videosource"),
@@ -391,6 +401,9 @@ void openMSXController::InitLaunchScript()
 	AddCommand(wxT("set gamma"),
 		[&](const wxString& c, const wxString& r) {
 			UpdateSetting(c, r); });
+	AddCommand(wxT("set noise"),
+		[&](const wxString& c, const wxString& r) {
+			UpdateSetting(c, r); });
 	AddCommand(wxT("set scanline"),
 		[&](const wxString& c, const wxString& r) {
 			UpdateSetting(c, r); });
@@ -415,21 +428,25 @@ void openMSXController::InitLaunchScript()
 	AddCommand(wxT("set speed"),
 		[&](const wxString& c, const wxString& r) {
 			UpdateSetting(c, r); });
+	AddCommand(wxT("set fastforwardspeed"),
+		[&](const wxString& c, const wxString& r) {
+			UpdateSetting(c, r); });
+	AddCommand(wxT("set fastforward"),
+		[&](const wxString& c, const wxString& r) {
+			UpdateSetting(c, r); });
+	AddCommand(wxT("set fullspeedwhenloading"),
+		[&](const wxString& c, const wxString& r) {
+			UpdateSetting(c, r); });
 	AddCommand(wxT("set maxframeskip"),
 		[&](const wxString& c, const wxString& r) {
 			UpdateSetting(c, r); });
-	AddCommand(wxT("set minframeskip"),
-		[&](const wxString& c, const wxString& r) {
-			UpdateSetting(c, r); });
-	AddCommand(wxT("set throttle"),
-		[&](const wxString& c, const wxString& r) {
-			UpdateSetting(c, r); });
 	AddCommand(wxT("set cmdtiming"),
 		[&](const wxString& c, const wxString& r) {
 			UpdateSetting(c, r); });
 	AddCommand(wxT("machine_info pluggable"),
 		[&](const wxString&, const wxString& r) {
 			HandlePluggables(r); });
+	m_relaunch = m_launchScript.size(); // !!HACK!!
 	AddCommand(wxT("machine_info connector"),
 		[&](const wxString&, const wxString& r) {
 			HandleConnectors(r); });
@@ -445,11 +462,10 @@ void openMSXController::InitLaunchScript()
 	AddCommand(wxT(""),
 		[&](const wxString&, const wxString&) {
 			m_appWindow->m_miscControlPage->InitConnectorPanel(); });
-	m_relaunch = m_launchScript.size(); // !!HACK!!
 	AddCommand(wxT(""),
 		[&](const wxString&, const wxString&) {
 			m_appWindow->m_audioControlPage->InitAudioIO(); });
-	AddCommand(wxT("machine_info sounddevice"),
+	AddCommand(wxT("set catapult_result [dict create] ; foreach dev [machine_info sounddevice] { dict append catapult_result $dev [machine_info sounddevice $dev]} ; set catapult_result"),
 		[&](const wxString&, const wxString& r) {
 			HandleSoundDevices(r); });
 	AddCommand(wxT("set mute"),
@@ -464,11 +480,14 @@ void openMSXController::InitLaunchScript()
 			m_appWindow->m_sessionPage->SetCassetteMode(info.Last());
 		});
 	AddCommand(wxT("openmsx_update enable plug"));
-	AddCommand(wxT("openmsx_update enable unplug"));
+	AddCommand(wxT("openmsx_update enable connector"));
+	AddCommand(wxT("openmsx_update enable hardware"));
+	AddCommand(wxT("openmsx_update enable extension"));
 	AddCommand(wxT("openmsx_update enable status"));
 	AddCommand(wxT(""), // finish launch
 		[&](const wxString&, const wxString&) {
 			m_appWindow->OpenMSXStarted();
+			m_isRelaunching = false;
 		});
 
 	AddSetting(wxT("renderer"),
@@ -480,9 +499,9 @@ void openMSXController::InitLaunchScript()
 	AddSetting(wxT("scale_factor"),
 		[&](const wxString&, const wxString& v) {
 			UpdateCombo(v, wxT("ScalerFactorSelector")); });
-	AddSetting(wxT("accuracy"),
+	AddSetting(wxT("vsync"),
 		[&](const wxString&, const wxString& v) {
-			UpdateCombo(v, wxT("AccuracySelector")); });
+			UpdateToggle(v, wxT("VSyncButton"), S_CONVERT); });
 	AddSetting(wxT("videosource"),
 		[&](const wxString&, const wxString& v) {
 			UpdateCombo(v, wxT("VideoSourceSelector")); });
@@ -504,21 +523,27 @@ void openMSXController::InitLaunchScript()
 	AddSetting(wxT("gamma"),
 		[&](const wxString&, const wxString& v) {
 			UpdateIndicator(v, wxT("GammaIndicator")); });
+	AddSetting(wxT("noise"),
+		[&](const wxString&, const wxString& v) {
+			UpdateIndicator(v, wxT("NoiseIndicator")); });
 	AddSetting(wxT("scanline"),
 		[&](const wxString&, const wxString& v) {
 			UpdateIndicator(v, wxT("ScanlineIndicator")); });
 	AddSetting(wxT("speed"),
 		[&](const wxString&, const wxString& v) {
 			UpdateIndicator(v, wxT("SpeedIndicator")); });
-	AddSetting(wxT("minframeskip"),
+	AddSetting(wxT("fastforwardspeed"),
 		[&](const wxString&, const wxString& v) {
-			UpdateIndicator(v, wxT("MinFrameSkipIndicator")); });
+			UpdateIndicator(v, wxT("FastForwardSpeedIndicator")); });
 	AddSetting(wxT("maxframeskip"),
 		[&](const wxString&, const wxString& v) {
 			UpdateIndicator(v, wxT("MaxFrameSkipIndicator")); });
-	AddSetting(wxT("throttle"),
+	AddSetting(wxT("fastforward"),
 		[&](const wxString&, const wxString& v) {
-			UpdateToggle(v, wxT("MaxSpeedButton"), S_INVERT | S_EVENT); });
+			UpdateToggle(v, wxT("FastForwardButton"), S_EVENT); });
+	AddSetting(wxT("fullspeedwhenloading"),
+		[&](const wxString&, const wxString& v) {
+			UpdateToggle(v, wxT("FastLoadingButton"), S_EVENT); });
 	AddSetting(wxT("power"),
 		[&](const wxString&, const wxString& v) {
 			UpdateToggle(v, wxT("PowerButton")); });
@@ -553,11 +578,11 @@ void openMSXController::InitLaunchScript()
 			wxString channel = n.Mid(0, n.Length() - 8); // remove "_balance"
 			m_appWindow->m_audioControlPage->SetChannelMode(channel, v);
 		});
-	AddSetting(wxT("msx-midi-in"),
+	AddSetting(wxT("*-in"),
 		[&](const wxString& n, const wxString& v) {
 			m_appWindow->m_audioControlPage->UpdateMidiPlug(
 				wxT("MidiInSelector"), v); });
-	AddSetting(wxT("msx-midi-out"),
+	AddSetting(wxT("*-out"),
 		[&](const wxString& n, const wxString& v) {
 			m_appWindow->m_audioControlPage->UpdateMidiPlug(
 				wxT("MidiOutSelector"), v); });
@@ -631,6 +656,11 @@ void openMSXController::AddSetting(
 
 void openMSXController::ExecuteStart(int startLine)
 {
+	if (m_isRelaunching && (startLine > 0)) {
+		// Canceling relaunch, already relaunching
+		return;
+	}
+	m_isRelaunching = (startLine > 0);
 	sendStep = startLine;
 	ExecuteNext();
 }
@@ -639,13 +669,13 @@ void openMSXController::ExecuteNext()
 {
 	WriteCommand(m_launchScript[sendStep].command,
 		[&](const wxString& c, const wxString& r) {
-			HandleLauch(c, r, true); },
+			HandleLaunch(c, r, true); },
 		[&](const wxString& c, const wxString& r) {
-			HandleLauch(c, r, false); });
+			HandleLaunch(c, r, false); });
 	m_launchCallback.push_back(m_launchScript[sendStep].callback);
 }
 
-void openMSXController::HandleLauch(const wxString& command, const wxString& result, bool ok)
+void openMSXController::HandleLaunch(const wxString& command, const wxString& result, bool ok)
 {
 	// Handle received command
 	assert(!m_launchCallback.empty());
@@ -808,37 +838,36 @@ void openMSXController::HandleConnectors(const wxString& result)
 {
 	m_connectors = utils::parseTclList(result);
 	m_connectorclasses.Clear();
+	m_connectorpluggeds.Clear();
 	for (auto& con : m_connectors) {
 		WriteCommand(wxT("machine_info connectionclass ") +
 		             utils::tclEscapeWord(con),
 			[&](const wxString&, const wxString& r) {
 				m_connectorclasses.Add(r); });
+		WriteCommand(wxT("plug ") + utils::tclEscapeWord(con),
+			[&](const wxString&, const wxString& r) {
+				int pos = r.Find(wxT(": "));
+				wxString pluggable = r.Right(r.Length() - pos - 2); // remove "connector: "
+				m_connectorpluggeds.Add(pluggable); });
 	}
 }
 
 void openMSXController::HandleSoundDevices(const wxString& result)
 {
-	auto devices = utils::parseTclList(result);
-	for (auto& dev : devices) {
-		dev = utils::tclEscapeWord(dev);
-	}
 	m_appWindow->m_audioControlPage->DestroyAudioMixer();
 	m_appWindow->m_audioControlPage->InitAudioChannels();
-	for (auto& dev : devices) {
-		WriteCommand(wxT("machine_info sounddevice " + dev),
-			[&](const wxString& c, const wxString& r) {
-				auto tokens = utils::parseTclList(c);
-				assert(tokens.GetCount() == 3);
-				m_appWindow->m_audioControlPage->AddChannelType(tokens[2], r);
-			});
+	auto devInfos = utils::parseTclList(result);
+	assert((devInfos.size() % 2) == 0);
+	for (auto it = devInfos.begin(); it != devInfos.end(); it += 2) {
+		m_appWindow->m_audioControlPage->AddChannelType(*it, *(it + 1));
 	}
-	WriteCommand(wxT(""),
-		[&](const wxString&, const wxString&) {
-			m_appWindow->m_audioControlPage->SetupAudioMixer(); });
+	m_appWindow->m_audioControlPage->SetupAudioMixer();
+
 	WriteCommand(wxT("set master_volume"),
 		[&](const wxString& c, const wxString& r) {
 			UpdateSetting(c, r); });
-	for (auto& dev : devices) {
+	for (auto it = devInfos.begin(); it != devInfos.end(); it += 2) {
+		auto dev = utils::tclEscapeWord(*it);
 		WriteCommand(wxT("set ") + dev + wxT("_volume"),
 			[&](const wxString& c, const wxString& r) {
 				UpdateSetting(c, r); });
@@ -914,23 +943,21 @@ bool openMSXController::Launch(wxString cmdline)
 	CreatePipes(&hInputRead, &hOutputWrite, &hErrorWrite, &hOutputRead, &hErrorRead);
 
 	DWORD dwProcessFlags = CREATE_NO_WINDOW | CREATE_DEFAULT_ERROR_MODE | CREATE_SUSPENDED;
-	WORD wStartupWnd = SW_HIDE;
 
 	STARTUPINFO si;
-	DWORD dwFlags = STARTF_USESHOWWINDOW | STARTF_USESTDHANDLES;
+	DWORD dwFlags = STARTF_USESTDHANDLES;
 	ZeroMemory(&si, sizeof(si));
 	si.cb = sizeof(si);
 	si.dwFlags = dwFlags;
 	si.hStdInput = hInputRead;
 	si.hStdOutput = hOutputWrite;
 	si.hStdError  = hErrorWrite;
-	si.wShowWindow = wStartupWnd;
 
 	// Note: m_pipeName is set by CreateControlParameter().
 	m_namedPipeHandle = CreateNamedPipe(m_pipeName, PIPE_ACCESS_OUTBOUND, PIPE_TYPE_BYTE, 1, 10000, 0, 100, nullptr);
 	if (m_namedPipeHandle == INVALID_HANDLE_VALUE) {
 		wxMessageBox(wxString::Format(
-			wxT("Error creating pipe: %ld"), GetLastError()));
+			wxT("Error creating pipe: %ld"), long(GetLastError())));
 		return false;
 	}
 
@@ -1034,7 +1061,7 @@ bool openMSXController::CreatePipes(
 
 void openMSXController::ShowError(const wxString& msg)
 {
-	wxMessageBox(msg + wxString::Format(wxT(": error %ld"), GetLastError()));
+	wxMessageBox(msg + wxString::Format(wxT(": error %ld"), long(GetLastError())));
 }
 
 void openMSXController::CloseHandles(
